@@ -1,3 +1,21 @@
+#######################################################################
+# rEMM - Extensible Markov Model (EMM) for Data Stream Clustering in R
+# Copyrigth (C) 2011 Michael Hahsler
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License along
+# with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+
 
 setMethod("transition", signature(x = "TRACDS", 
 		from = "matrix", to = "missing"),
@@ -36,7 +54,7 @@ setMethod("transition_matrix", signature(x = "TRACDS"),
 		type <- match.arg(type)
 
 		## get transition count matrix
-		m <- smc_countMatrix(x@mm)
+		m <- smc_countMatrix(x@tracds_d$mm)
 		
 		if(plus_one) m <- m+1
 
@@ -63,7 +81,7 @@ setMethod("initial_transition", signature(x = "TRACDS"),
 		type=c("probability", "counts", "log_odds"), plus_one = FALSE){
 		type <- match.arg(type)
 
-		ic <- smc_initialCounts(x@mm)
+		ic <- smc_initialCounts(x@tracds_d$mm)
 		if(plus_one) ic <- ic+1
 
 		switch(type,
